from qgis.PyQt.QtWidgets import QDialog
from ..tools.settings import set_setting
from qgis.core import QgsApplication
from ..tools.networking import NetworkRequestTask


class SettingsPanel():
    def __init__(self, dialog: QDialog, plugin) -> None:
        self._plugin = plugin
        self._dialog = dialog
        self.api_button_enabled = True

    def setup_panel(self) -> None:
        print("Setup Settings panel")
        self.display_error()
        self._dialog.settings_api_key_line_edit.setText(self._plugin.api_key)
        self._dialog.settings_api_key_line_edit.textChanged.connect(self.api_key_changed)
        self._dialog.settings_text_browser.setOpenExternalLinks(True)

        self._dialog.settings_get_api_key_button.clicked.connect(self.get_api_key)

    def get_api_key_callback(self, response):
        if response.status_code == 200:
            print("Send")
            self._dialog.settings_get_api_key_button.setText("API key has been send. Click here to send again")
            self.api_button_enabled = True
        else:
            print("Error form server: ", response.status_code)
            self._plugin.error_message = 'Server error'
            self.display_error()
            self._dialog.settings_get_api_key_button.setText("Get API key")
            self.api_button_enabled = True

    def get_api_key(self):
        if self.api_button_enabled:
            if self._dialog.settings_api_key_terms.isChecked():

                email = self._dialog.settings_api_key_email.text()

                self._plugin.error_message = None
                self.display_error()

                self._dialog.settings_get_api_key_button.setText("Sending... Please wait")
                self.api_button_enabled = False
          

                # Make request
                url = "https://api-key-robot.stratopo.nl/"
                requestBody = { "emailaddress": email }
                headers = { 'Content-type': 'application/json' }

                globals()['get_api_key_task'] = NetworkRequestTask(url, headers, requestBody, self.get_api_key_callback)
                QgsApplication.taskManager().addTask(globals()['get_api_key_task'])
                
            else:
                self._plugin.error_message = 'Please accept the Terms and Conditions and the Privacy Policy'
                self.display_error()
        
            

    def make_active_panel(self):
        print('settings make active')
        self.display_error()

    def display_error(self):
        if self._plugin.error_message is None:
            self._dialog.settings_error_label.setVisible(False)
        else:
            self._dialog.settings_error_label.setText(self._plugin.error_message)
            self._dialog.settings_error_label.setVisible(True)

    def api_key_changed(self): 
        self._plugin.api_key = self._dialog.settings_api_key_line_edit.text()
        set_setting("api_key", self._plugin.api_key)
        print("Key saved")